'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.resolver = exports.generator = undefined;

var _generator = require('./generator');

Object.defineProperty(exports, 'generator', {
    enumerable: true,
    get: function () {
        return _generator.generator;
    }
});

var _schemaResolver = require('./schema/schema-resolver');

Object.defineProperty(exports, 'resolver', {
    enumerable: true,
    get: function () {
        return _schemaResolver.resolver;
    }
});
exports.server = server;

require('babel-polyfill');

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _serverDestroy = require('server-destroy');

var _serverDestroy2 = _interopRequireDefault(_serverDestroy);

var _store = require('./store');

var _server = require('./server');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DEFAULT_HOST = 'localhost';
const DEFAULT_PORT = 3000;

function server() {
    let config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    if (config.quiet) {
        console.log = function () {};
    }

    console.log();
    console.log(_chalk2.default.cyan('  \\Starting!'));

    console.log();
    console.log(_chalk2.default.gray('  Start init db', config.source || 'Default path: 当前目录'));

    let db = (0, _store.store)(config);

    console.log(_chalk2.default.gray('  db inited'));

    let app = (0, _server.create)(db, config);
    let server = app.listen(config.port || DEFAULT_PORT, config.host || DEFAULT_HOST);

    (0, _serverDestroy2.default)(server);

    prettyPrint(config, db.getState(), config.routes);
}

function prettyPrint() {
    let config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    let db = arguments[1];
    let routes = arguments.length <= 2 || arguments[2] === undefined ? [] : arguments[2];

    let host = config.host ? config.host : DEFAULT_HOST;
    let port = config.port || DEFAULT_PORT;
    let root = `http://${ host }:${ port }`;

    console.log();
    console.log(_chalk2.default.bold('  Resources'));
    for (let key of Object.keys(db)) {
        console.log(`  ${ root }/${ key }`);
    }

    if (routes) {
        console.log();
        console.log(_chalk2.default.bold('  Other routes'));
        routes.forEach(_ref => {
            let url = _ref.url;
            return console.log(`  rule ->  ${ url }`);
        });
    }

    console.log();
    console.log(_chalk2.default.bold('  Home'));
    console.log('  ' + root);
    console.log();
}