'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.router = router;

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _plural = require('./plural');

var _singular = require('./singular');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function router(db, config) {
    let router = _express2.default.Router();
    router.db = db;

    router.render = function (request, response) {
        response.jsonp(response.locals.data);
    };

    router.get('/db', (request, response, next) => {
        response.locals.data = db.getState();
        next();
    });

    db.forEach((value, key) => {
        if (_lodash2.default.isPlainObject(value)) {
            router.use('/' + key, (0, _singular.singular)(db, key, config.singularOverrides));
            return;
        }

        if (_lodash2.default.isArray(value)) {
            router.use('/' + key, (0, _plural.plural)(db, key, config.pluralOverrides));
            return;
        }
    }).value();

    router.use((request, response) => {
        if (!response.locals.data) {
            response.status(404);
            response.locals.data = {};
        }

        router.render(request, response);
    });

    router.use((error, request, response, next) => {
        console.error(error.stack);
        response.status(500).send(error.stack);
    });

    return router;
}