'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.list = list;
exports.get = get;
exports.create = create;
exports.update = update;
exports.remove = remove;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// GET /name
// GET /name?keyword=
// GET /name?page=&pageSize=
// GET /name?order=&orderBy=
function list(db, name, request, response, next) {
    let chain = db.get(name);
    let totalCount = chain.size().value();

    // 参数
    let keyword = request.query.keyword;
    let order = request.query.order;
    let orderBy = request.query.orderBy;
    let page = request.query.page;
    let pageSize = request.query.pageSize;

    // 搜索
    if (keyword) {
        keyword = keyword.toLowerCase();
        chain = chain.filter(object => {
            for (let value of Object.values(object)) {
                return value && value.toString().toLowerCase().indexOf(keyword) >= 0;
            }
        });
    }

    // 排序
    if (order || orderBy) {
        order = order || 'asc';
        order = order.toLowerCase();

        chain = chain.sortBy(element => _lodash2.default.get(element, orderBy || 'id'));

        if (order === 'desc') {
            chain = chain.reverse();
        }
    }

    pageSize = Number.parseInt(pageSize, 10) || 50;
    page = Number.parseInt(page, 10) || 1;

    let start = (page - 1) * pageSize;
    let end = page * pageSize;
    chain = chain.slice(start, end);

    response.locals.data = { results: chain.value(), totalCount };
    next();
}

function get(db, name, request, response, next) {
    let id = request.params.id;
    response.locals.data = db.get(name).getById(id).value();
    next();
}

function create(db, name, request, response, next) {
    let resource = db.get(name).insert(_extends({}, request.body, { status: 1 })).value();

    response.status(201);
    response.locals.data = resource;
    next();
}

function update(db, name, request, response, next) {
    let id = request.params.id;
    let chain = db.get(name);
    chain = chain.updateById(id, request.body);
    // chain = request.method === 'PATCH' ? chain.updateById(id, request.body) : chain.replaceById(id, request.body);

    let resource = chain.value();
    if (resource) {
        response.locals.data = resource;
    }

    next();
}

function remove(db, name, request, response, next) {
    db.get(name).removeById(request.params.id).value();

    next();
}