'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.store = store;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _lowdb = require('lowdb');

var _lowdb2 = _interopRequireDefault(_lowdb);

var _underscoreDb = require('underscore-db');

var _underscoreDb2 = _interopRequireDefault(_underscoreDb);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

var _schemaScanner = require('./schema/schema-scanner');

var _schemaContainer = require('./schema/schema-container');

var _generator = require('./generator');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const STORE_PATH = 'db.json';
const DEFAULT_COUNT = 10;

/**
 * 初始化emock
 * 1. 扫描实体schema、api schema，并dref
 * 2. 随机生成实体数据
 * 3. 根据实体schema中的扩展字段建立关联关系
 *
 *
 * @method store
 * @param {Object} options emock配置
 * @param {string} options.source 存数mock数据的文件路径，相对process.cwd()
 * @param {string} options.constant 不变数据的文件路径，相对process.cwd()
 * @param {string} options.schemaDir schema所在目录
 * @param {number} options.count 默认生成的一个实体的数据条数
 */
function store(_ref) {
    var _ref$source = _ref.source;
    let source = _ref$source === undefined ? process.cwd() + STORE_PATH : _ref$source;
    let constant = _ref.constant;
    let schemaDir = _ref.schemaDir;
    let relations = _ref.relations;
    var _ref$count = _ref.count;
    let count = _ref$count === undefined ? DEFAULT_COUNT : _ref$count;

    schemaDir = _path2.default.isAbsolute(schemaDir) ? schemaDir : _path2.default.join(process.cwd(), schemaDir);
    let scanner = new _schemaScanner.SchemaScanner();
    // 从指定目录中扫描schema
    let schemas = scanner.scan(schemaDir);
    // 注册schema到容器中
    schemas.forEach(schema => _schemaContainer.container.registerEntity(schema.id, schema));

    // 如果已经有数据，就不再生成了，如果没有就生成
    let hasData = _fs2.default.existsSync(source);

    if (!_lodash2.default.isString(source)) {
        throw new Error('store文件路径必须是字符串');
    }

    let db = (0, _lowdb2.default)(source);
    db._.mixin(_underscoreDb2.default);

    if (!hasData) {
        initData(db, _schemaContainer.container, _generator.generator, count);
        addContantData(db, constant);
        createRelations(db, relations, _schemaContainer.container);
    }

    return db;
}

function initData(store, contianer, generator, count) {
    for (let _ref2 of contianer.getEntities().entries()) {
        var _ref3 = _slicedToArray(_ref2, 2);

        let id = _ref3[0];
        let schema = _ref3[1];

        let collectionId = _pluralize2.default.plural(id);
        store.set(collectionId, []).value();
        let instances = generator.create(schema.schema, count);
        instances.forEach(instance => store.get(collectionId).insert(instance).value());
    }
}

function addContantData(db, constant) {
    if (!constant) {
        return;
    }

    let constantData = JSON.parse(_fs2.default.readFileSync(_path2.default.join(process.cwd(), constant), 'utf-8'));
    for (let _ref4 of Object.entries(constantData)) {
        var _ref5 = _slicedToArray(_ref4, 2);

        let key = _ref5[0];
        let value = _ref5[1];

        db.set(key, value).value();
    }
}

function createRelations(db) {
    let relations = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    let container = arguments[2];

    for (let _ref6 of Object.entries(relations)) {
        var _ref7 = _slicedToArray(_ref6, 2);

        let entity = _ref7[0];
        let relation = _ref7[1];

        let entities = getResources(db, entity);
        // 如果没有实体数据，进行下一项
        if (!entities.length) {
            continue;
        }

        let schema = container.getEntity(entity);

        for (let _ref8 of Object.entries(relation)) {
            var _ref9 = _slicedToArray(_ref8, 2);

            let property = _ref9[0];
            let dependency = _ref9[1];

            // 非必需属性不作关联
            if (!schema.isRequired(property)) {
                continue;
            }

            let propertySchema = schema.getPropertySchema(property);
            let type = schema.getType(propertySchema);
            if (!type) {
                continue;
            }

            let relatedResources = getResources(db, dependency.resource);
            let relatedResourcesCount = relatedResources.length;

            entities = entities.map(entity => {
                if (type === 'string' || type === 'integer' || type === 'number') {
                    let index = Math.floor(Math.random() * relatedResourcesCount);
                    return _extends({}, entity, {
                        [property]: relatedResources[index][dependency.field]
                    });
                } else if (type === 'array') {
                    let selectedResources = _lodash2.default.shuffle(relatedResources).slice(0, entity[property] && entity[property].length);
                    return _extends({}, entity, {
                        [property]: _lodash2.default.map(selectedResources, dependency.field)
                    });
                }
            });
        }

        setResources(db, entity, entities);
    }
}

function getResourceName(db, entity) {
    return db.has(entity).value() ? entity : (0, _pluralize2.default)(entity);
}

function getResources(db, entity) {
    let resourceName = getResourceName(db, entity);
    return resourceName === entity ? [db.get(resourceName).value()] : db.get(resourceName).value();
}

function setResources(db, entity, entities) {
    let resourceName = getResourceName(db, entity);
    resourceName === entity ? db.set(resourceName, entities[0]).value() : db.set(resourceName, entities).value();
}